/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __HypFiller__
#define __HypFiller__

#include "ChartParserHyp.hh"
#include "CombinationBeam.hh"

// This class provides the core functionality of applying
// an incomplete rule to completed equivalence class chart items
// and creating new conseqent items. It includes tools static
// methods to compute LM probabilities as well. These
// static functions are called by ChartParserHyp's contructor
class HypFiller {

 public:


  // Takes a CChartParserHypComplete* and files this hyp into the
  // chart by locating the appropriate cell
  static bool AddCompletedHyp(
      CChartParserHypComplete* pHyp,
      bool addAll,
      CCompleteHypChartCell* chartCell,
      CCoarseGrainedEq* newlyCreatedNodes
      );

  // Take the list of rules that match this one source side span +
  // the back pointers that each rule points to, and instantiate
  // the top K real CChartParserNode based complete hyps and file
  // them via AddCompletedHyp.
  // Returns a pair of int's representing the number expanded,
  // and the number that could have been expanded.
  static pair<int, int> SpawnKBestCompleteHyps(
      RulesByProductionResult& hypRuleCombinations,
      bool addAll,
      CCompleteHypChartCell* chartCell,
      CCoarseGrainedEq* newlyCreatedNodes
      );

 protected:

  // The ComboBeam is a priority queue that manages the generation
  // of new hyps due to the completion of an incomplete hyp.
  // The CreateInitialTuples function initializes this search
  // by with first best options from each production node,
  // and fills the initialTuples vector. As a side effect, it
  // also queues all the DummyLookup(s) required by this
  // rule application
  static void CreateInitialTuplesAndQueueLookups(
    RulesByProductionResult::iterator begin,
    RulesByProductionResult::iterator end,
    vector<CCombinationTuple*>* initialTuples,
    int& allPossibleExpansions,
    bool addAll);

  // do spawning for a particular production result only -
  // verify that all combos do produce the same production result
  // returns a pair of int's representing the number expanded, and
  // the number that could have been expanded
  static pair<int, int> SpawnForProductionResult(
    RulesByProductionResult::iterator begin,
    RulesByProductionResult::iterator end,
    bool addAll,
    CCompleteHypChartCell* chartCell,
    CCoarseGrainedEq* newlyCreatedNodes
    );

  // Displays the rule with its CProductionNode backpointers
  static void DisplayInstantiatedRule(
      CRule* pRule,
      CChartParserHypIncomplete* pIncompleteHyp);
};

#endif  // __HypFiller__
