/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __METRICRAW__
#define __METRICRAW__

#include <iostream>



//! Size of the n-grams being tracked. Dont use this variable outside MetricRaw, use MetricSearch->GetNGramsConsidered
#define NSZ 5

// a n-gram tracker
typedef double Counter[NSZ];

//holds intermediate metric statistics
class RawCounts;

class RawCounts{

public:
  RawCounts(){
    for (int i=0; i<NSZ; i++){
      numCorrect[i] = 0;
      numSuggested[i] = 0;
    }
    referenceLengths = 0;
  }
  RawCounts(const RawCounts& r){
    for (int i=0; i<NSZ; i++){
      numCorrect[i] = r.numCorrect[i];
      numSuggested[i] = r.numSuggested[i];
    }
    referenceLengths = r.referenceLengths;
  }

  Counter numCorrect;
  Counter numSuggested;
  double referenceLengths;

  RawCounts operator-(const RawCounts& d2) const{
    RawCounts r;
    for (int i=0; i<NSZ; i++){
      r.numCorrect[i] = numCorrect[i]-d2.numCorrect[i];
      r.numSuggested[i] = numSuggested[i]-d2.numSuggested[i];
    }
    r.referenceLengths = referenceLengths-d2.referenceLengths;
    return r;
  }

  RawCounts operator + (const RawCounts& d2) const{
    RawCounts r;
    for (int i=0; i<NSZ; i++){
      r.numCorrect[i] = numCorrect[i]+d2.numCorrect[i];
      r.numSuggested[i] = numSuggested[i]+d2.numSuggested[i];
    }
    r.referenceLengths = referenceLengths+d2.referenceLengths;
    return r;
  }

  RawCounts& operator = (const RawCounts& r){
    if (this == &r){
      return *this;
    }
    for (int i=0; i<NSZ; i++){
      this->numCorrect[i] = r.numCorrect[i];
      this->numSuggested[i] = r.numSuggested[i];
    }
    this->referenceLengths = r.referenceLengths;

    return *this;
  }

};

//ostream& operator << (ostream& os, const RawCounts* s);
//istream& operator >> ( istream& in, RawCounts &s );

#endif
