/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __OPTIMIZE_HH__
#define __OPTIMIZE_HH__

#include "MetricSearch.hh"
#include "MetricRaw.hh"
#include <vector>

//!Parameters that multiply the respective feature
typedef std::vector<double> Lambdas;

//!Feature data for each candidate, as float since they need to be stored in memory
typedef std::vector<float> Features;

//!The optimization metric is initialized against a particular metric
class COptimize{
public:

  //default, no metric
  COptimize();

  //!Initialize an optimizer against a metric
  COptimize(MetricSearch* pMetric){
    m_pMetric = pMetric;
  }

  void SetMetric(MetricSearch* pMetric) {
    m_pMetric = pMetric;
  }

  //! Find parameters that use argmin to maximize metric score (this version works with doubles for the starting params)
  virtual Lambdas minModelCostMaxMetricScore(const std::vector<Features*>& costs, const std::vector<RawCounts*>&  scores, const std::vector<int>& startEnd, const Lambdas& initialValues, int firstXsentences)=0;

protected:

  //!The error metric that determines the metric score
  MetricSearch* m_pMetric;
};


#endif
