/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#include "ParserInfo.hh"
#include <limits.h>
#include <assert.h>


//============================================================================
// static initialization modified by FastTranslateChart
//============================================================================
//! For CChartParserHyp
vector<double> CChartParserInfo::m_scalingFactors = vector<double>();

//! For CChartParser
bool CChartParserInfo::m_GlueAndEOSAssumptions = true;
double CChartParserInfo::FutureCostImpact = 0.0;

int CChartParserInfo::m_sourceLength = 0;
CVocabulary* CChartParserInfo::m_pVocabulary = NULL;


int CChartParserInfo::BeamSizeFactor = 10;
int CChartParserInfo::m_recomputeLMCostsDuringKBest=0;
int CChartParserInfo::m_rescoreLM = 0;
int CChartParserInfo::C_HistoryLength = 2;
int CChartParserInfo::C_SRI_HistoryLength = 2;

bool CChartParserInfo::m_compareTargetWords = true;
bool CChartParserInfo::m_lmInCoarse = true;


//CSentence* CChartParserInfo::m_pSentence = NULL;
CWordSequence CChartParserInfo::m_pSentence = CWordSequence();

int CChartParserInfo::MAX_HYP_COUNT = 100;
//int CChartParserInfo::m_combinationCount = 0; //!< this one is initialized in InitSearch
int CChartParserInfo::m_maxCombinationCount = 0;
CWordIndex CChartParserInfo::m_unknownWI = -1;
CWordIndex CChartParserInfo::m_bosWI = -1;
CWordIndex CChartParserInfo::m_eosWI = -1;
CWordIndex CChartParserInfo::m_lmBoundaryWI = -1;
CWordIndex CChartParserInfo::m_FullSentenceNT = 0;
CWordIndex CChartParserInfo::m_GlueNT = 0;
int CChartParserInfo::m_stackLevel = 0;
bool CChartParserInfo::m_noTranslationOutputOfUnknownWords = false;
//nbest options
bool CChartParserInfo::m_extractUnique = true;
int CChartParserInfo::m_extractRecombine = 1; //!< Re-combine items as you do the n-best extraction, only applicable in cmp=0
int CChartParserInfo::m_nbestStyle = 2; //!< By default use the HC05 search

double CChartParserInfo::m_aStarBeam = 0.8; //! fuzz factor for termination of A* searhc
double CChartParserInfo::m_astarFuzz = 0.0; //! fuzz factor to terminate search

//for Pruning
NTPruningMap CChartParserInfo::m_pruningMap = NTPruningMap();
TagSet CChartParserInfo::m_tagsForLM = TagSet();
double CChartParserInfo::m_maxCostDifferencePerCell; //beam perc for production nodes
bool CChartParserInfo::m_pruningMapUseBoundary = false;


//combo pruning
float CChartParserInfo::m_comboPruningBeamSize; //!< used in the spawning process
float CChartParserInfo::m_comboPruningMinBeamSize; //!< used in the spawning process
float CChartParserInfo::m_comboPruningFuzzCostDifference; //!< used in the spawning process

bool CChartParserInfo::m_comboPruningGlobal=1; //!< does combo pruning across all incomplete hyps
bool CChartParserInfo::m_comboPruningUseLM=1; //!< use the LM during combo pruning
int CChartParserInfo::m_updateBeamSize=0; //!< update the beam size to allow BeamSizeFactor active hyps
//bool CChartParserInfo::m_comboPruningUseLexical; //!< use the Lexical hyps in the same beam during combo pruning
//rule app count operations to simulate how it used to work before (these are really bools, but the params handles stuff strangely)
int CChartParserInfo::m_updateHypRuleAppCount=1; //update the hyp's rule app count with the fine-grained eq class counts
int CChartParserInfo::m_lowestRuleAppCount=0; //use the lowest rule app count instead of the count of the lowest cost hyp
int CChartParserInfo::m_multiplicativeRA=0; //use the rule app count as a multiplicative factor
//system operation
int CChartParserInfo::m_verbosity=0; //!< how verbose is the output (0-1-2)

int CChartParserInfo::m_saveAllAlternatives=1; //! if true, then we save all alternatives in the eq class (uses much more memory)
int CChartParserInfo::m_breadthAStar=1; //run the fast way
int CChartParserInfo::SAMTLM_UseBERFormat=0;
int CChartParserInfo::m_useSpanFeatures=0;
int CChartParserInfo::m_normalizeReferences = 0; //! Normalize references with the normalizing script


//decoding statistics -- see SpawnKBestCompleteHyps for details
int CChartParserInfo::m_hypsAdded=0;
double CChartParserInfo::m_spawntimecpu = 0;

int CChartParserInfo::m_hypsDeleted=0; //deleted after completion in PruneUnfiledHyps
int CChartParserInfo::m_hypsDeletedThreshold = 0; //total number of hyps deleted when finalizing the node due to threshold
int CChartParserInfo::m_hypsDeletedHistogram = 0; //total number of hyps deleted when finalizing the node due to histogram

int CChartParserInfo::m_numTopCalls =0;
int CChartParserInfo::m_numTuplesProposed =0;  //number of tuples that the combo beam had to calculate costs for
int CChartParserInfo::m_allPossibleExpansions=0;
int CChartParserInfo::m_numAddedPostFailure = 0;
int CChartParserInfo::m_childBetter =0;

//kbest stats
int CChartParserInfo::m_noCollisions=0;
int CChartParserInfo::m_uniqueCollisions=0;
int CChartParserInfo::m_betteredCollisions=0;
int CChartParserInfo::m_worseCollisions=0;
int CChartParserInfo::m_proposedAddedKBest=0;
int CChartParserInfo::m_topCallsKBest=0;
int CChartParserInfo::m_deletedKBest=0;
int CChartParserInfo::m_numUnique=0;
int CChartParserInfo::m_globalLMCache=0;
int CChartParserInfo::m_dontReloadBeam=0;

uint64_t CChartParserInfo::m_lmCall=0;
uint64_t CChartParserInfo::m_lmCacheHit=0;

// Set directly as parameters
string  CChartParserInfo::UnknownWord;
string CChartParserInfo::FullSentNonTerminal;
string CChartParserInfo::NormalizingScript;
bool CChartParserInfo::RemoveUnt;
int CChartParserInfo::HypsPerNode;
int CChartParserInfo::MaxCombinationCount;
int CChartParserInfo::MaxRuleAppCountDifference;
int CChartParserInfo::CompareTargetWords;

string CChartParserInfo::TagSetList;
string CChartParserInfo::ComboPruningBeamSize;
string CChartParserInfo::ComboPruningMinBeamSize;
string CChartParserInfo::ComboPruningFuzzCostDifference;
bool CChartParserInfo::ComboPruningGlobal;
bool CChartParserInfo::ComboPruningUseLM;
string CChartParserInfo::MaxCostDifferencePerCell;
string CChartParserInfo::AStarBeam;
string CChartParserInfo::PruningMap;
bool CChartParserInfo::PruningMapUseBoundary;
int CChartParserInfo::NGramsConsidered;
int CChartParserInfo::NBest;
int CChartParserInfo::DisplayNBestData;
bool CChartParserInfo::GenOracleScores;
int CChartParserInfo::IterationLimit;
bool CChartParserInfo::ExtractUnique;
int CChartParserInfo::Verbosity;
int CChartParserInfo::NormalizeReferences;
bool CChartParserInfo::ExtractRecombine;
int CChartParserInfo::NBestStyle;
int CChartParserInfo::RescoreLM;
int CChartParserInfo::RecomputeLMCostsDuringKBest;
bool CChartParserInfo::RA_UpdateHyp;
bool CChartParserInfo::RA_LowestCount;
bool CChartParserInfo::RA_Mult;
bool CChartParserInfo::GlobalLMCache;

bool CChartParserInfo::UseSpanFeatures;
int CChartParserInfo::BreadthAStar;
bool CChartParserInfo::DontReloadBeam;
bool CChartParserInfo::SaveAllAlternatives;
bool CChartParserInfo::UpdateBeamSize;
int CChartParserInfo::HistoryLength;

NTOccurrenceStatisticsMap CChartParserInfo::m_nonterminalOccurrenceStatsMap = NTOccurrenceStatisticsMap();

void CChartParserInfo::LoadTagsForLM(const char* options){
  cout<<"Processing TagSet: "<<options<<endl;
  samtstring optionsString = options;
  char* s = strtok((char*)optionsString.c_str(), ",");
  while (s!=NULL){
    CWordIndex nt = CChartParserInfo::m_pVocabulary->GetIndex(s);
    CChartParserInfo::m_tagsForLM.insert(nt);
    cout<<"AddingTag: "<<s<<" ";
    s = strtok(NULL, ",");
  }
  cout<<endl;

}

float CChartParserInfo::StringToFloat(samtstring s){
  return StringToFloat(s.c_str());
}

float CChartParserInfo::StringToFloat(const char* s){

    double beta;
    if (strcmp(s, "inf")==0){
      beta = FLT_MAX/10; // very high number for infinity
    } else {
      beta = atof(s);
    }
    return beta;
}


void CChartParserInfo::LoadPruningMap(const char* options){
  cout<<"Processing PruningMap: "<<options<<endl;
  samtstring optionsString = options;
  char* s = strtok((char*)optionsString.c_str(), "-");
  while (s!=NULL){
    //get the non-terminal - unless its -1
    CWordIndex nt;
    if (strcmp(s, "0")==0){
      //the generic non-terminal
      nt=0;
    }else{
      //cout<<"Getting Index: "<<s<<endl;
      //cout<<"Vocab:"<<CChartParserInfo::m_pVocabulary<<endl;
      nt = CChartParserInfo::m_pVocabulary->GetIndex(s);
      //az: the following will never happen as GetIndex creates indices for words on the fly
      //      if (nt==CVocabulary::UNK){
      //	cout<<"Unknown Non-Terminal in LoadPruningMap: "<<s<<endl;
      //	exit(-1);
      //      }
    }

    cout<<"AddingPruningOption: "<<s;
    //load the next 2 elements
    s = strtok(NULL, "-");
    double b = atof(s);

    s = strtok(NULL, "-");
    double beta;
    if (strcmp(s, "inf")==0){
      beta = FLT_MAX/10; // very high number for infinity
    } else {
      beta = atof(s);
    }
    cout<<" b="<<b<<" beta="<<beta<<endl;


    m_pruningMap[nt] = pair<double, double>(b, beta);

    //move it forward
    s = strtok(NULL, "-");
  }
}




void CChartParserInfo::PrintParsingOutput(const CWordSequence& v, ostream& os){
  //!Static utility method that avoids having to write this loop every time
  for (CWordSequence::const_iterator iter=v.begin(); iter!= v.end(); ++iter){
      if ((*iter)<=0){
	os <<*iter<<" ";
      }else{
	
	if (CChartParserInfo::m_pVocabulary->GetIndex("#(") == *iter) {
	  os << "( ";
	  assert(iter!=v.end());
	  ++iter;
	  PrintParsingOutput_SafePrint(*iter, os);
	  // also, treat the next two elements as numbers, not word indices (because they represent the source span)
	  assert(iter!=v.end());
	  ++iter;
	  os<<"_" << *iter;
	  assert(iter!=v.end());
	  ++iter;
	  os<< "-" << *iter;
	  assert(iter!=v.end());
	  ++iter;
	  while (*iter != CChartParserInfo::m_pVocabulary->GetIndex("#") ) {
	    os << "_";
	    PrintParsingOutput_SafePrint(*iter, os);
	    ++iter;
	  }
	} else {
	  PrintParsingOutput_SafePrint(*iter, os);
	}
	os<<" ";
      }
  }
}

//! Sub of PrintParsingOutput
void CChartParserInfo::PrintParsingOutput_SafePrint(const int i, ostream& os){
  if (i == CChartParserInfo::m_pVocabulary->GetIndex("#(") ) {
    os << "(";
  } else if (i == CChartParserInfo::m_pVocabulary->GetIndex("#)") ) {
    os << ")";
  } else if (i == CChartParserInfo::m_pVocabulary->GetIndex("(") ) {
    os << "#LeftParen";
  } else if (i == CChartParserInfo::m_pVocabulary->GetIndex(")") ) {
    os << "#RightParen";
  } else if (i == CChartParserInfo::m_pVocabulary->GetIndex("\"") ) {
    os << "#QuotationMark";
  } else {
    os << CChartParserInfo::m_pVocabulary->GetWord(i);
  }
}



//!Static utility method that avoids having to write this loop every time
void CChartParserInfo::PrintVocabSequence(const CWordSequence& v, ostream& os){
  for (CWordSequence::const_iterator iter=v.begin(); iter!= v.end(); ++iter){
    if ((*iter)<=0){
      os << *iter;
    }else{
	//      cout<<" ("<<m_pVocabulary->GetWord(*iter)<<") ";
      if (m_pVocabulary!=NULL){
	os<<m_pVocabulary->GetWord(*iter);
      }else{
	os<<*iter;
	}
    }
    //if not end
    if (iter!=v.end()-1){
	os<<" ";
    }
  }
}
