/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef Rule_hh
#define Rule_hh

#include <vector>
#include <list>
#include "Vocabulary.hh"


typedef vector<CWordIndex> CWordSequence;

class CRule {

public:

  CWordIndex m_productionResult;

  CWordSequence m_translationOutput;

  std::vector<float> m_negLogProbs;

  samtstring m_rule;

  static CVocabulary* pVoc; //! statically shared vocab for all rules

  static int C_NumModelsTM; //number of tm models represented in each hyp:
  //glue feature, word count, phrase count (1),
  //RF|Result, RF|Target, RF|Source,
  //OccurrenceCount, ...
  static int C_NumModelsTotal; //!number of models represented in each hyp (LM + TMs)
  static int C_NumLMs; //! number of language models
  static int C_LMLocation; //!location of the forst LM in the list of models
  static int C_GlueLocation; //!location of the Glue feature
  static int C_RuleApplicationCountLocation; //! location of the rule-application count feature
  static int C_SourceSpanPenaltyLocation; //! location of the source span penalty feature if --UseSpanFeatures=1

  //!Constructor to build a rule from scratch
  CRule(const samtstring& strkey, const samtstring& strdata);

  //!Copy constructor
  CRule(const CRule& r);

  //!Assignment operator
  CRule& operator = (const CRule& r);

  //!Utilty display method for debugging
  void Display();

  //static void Tokenize(const string& str, std::vector<samtstring>& tokens, const string& delimiters = " ");

  static void Tokenize(const samtstring& str,
		       vector<samtstring>& tokens,
		       const samtstring& delimiters=" ")
  {
    // Skip delimiters at beginning.
    samtstring::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    samtstring::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (samtstring::npos != pos || samtstring::npos != lastPos)
      {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
      }
  }

  static void ParseDoubleSequence(samtstring floatSequenceStr, vector<double>& doubleVector) {
    ParseDoubleSequence(floatSequenceStr.c_str(), doubleVector);
  }
  static void ParseDoubleSequence(const char* floatSequenceStr, vector<double>& doubleVector) {
    doubleVector.clear();
    vector<samtstring> tmpscf;
    CRule::Tokenize(floatSequenceStr,tmpscf,",_");
    for (int i = 0; i<tmpscf.size(); i++) {
      double f = atof(tmpscf[i].c_str());
      cout<<f<<" ";
      doubleVector.push_back(f);
    }
  }

};



#endif
