/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __RULESET_HH__
#define __RULESET_HH__

#include "Rule.hh"

// Result returned by a RuleSet query
// pair.first : least-significant bit 1 iff there was an exact match
//              and pair.second is an iterator over exact match rules
// pair.first : second least-second bit 1 iff there was a strict prefix match
typedef pair< short,vector<CRule> > RuleQueryResult;
//! define bit constants for the type of match in CBTreeResult.first
#define C_ExactMatch 1
#define C_StrictPrefixMatch 2


// RuleSet provides an interface to a set of rules (translation model)
// which are applied to translate source sentences. This class provides
// the interface to lookup these rules from source source (DB/file etc.)
class RuleSet {
 public:
  RuleSet(){}
  virtual RuleQueryResult QueryRuleSet(const vector<CWordIndex>& prefix) = 0;
};

#endif // __RULESET_HH__
