/*
  Syntax Based Statistical Machine Translation
  Chart Parsing Tool for Generalized, Annotated Synchronous Grammars
  Copyright (C) 2006  Andreas Zollmann, Ashish Venugopal
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __RunDecoder__
#define __RunDecoder__

#include "FastLatticeChartParser.hh"
#include "pstream.h"

// Top level class that takes as input a TM and an LM and run translation on a
// development set.
// This class assumes static variables have been set appropriately in
// CChartParserInfo.
class RunDecoder {
 public:

  void InitializeDecoder(RuleSet* rules,
                         const vector<CLanguageModel*>& lms,
                         const vector<double>& Weights);

  void RunMERonDev( samtstring ScoringMetric,
                    samtstring ReferenceList,
                    samtstring GainFile,
                    int NumReferences,
                    COptimize* pOptimizer,
                    vector<CWordSequence>& sourceCorpus,
                    vector< vector<double> >& featureRange );

  void ShutdownDecoder();

 protected:
  redi::pstream pbuf;
  bool m_useNormalizing;
  vector<double> ScalingFactors;
  CChartParser* parser;
};

#endif // __RunDecoder__
